@@ -17,6 +17,7 @@ import proto  # type: ignore
 
 from google.cloud.datastore_v1.types import entity
 from google.cloud.datastore_v1.types import query as gd_query
+from google.protobuf import timestamp_pb2  # type: ignore
 
 
 __protobuf__ = proto.module(
@@ -58,9 +59,20 @@ class LookupRequest(proto.Message):
             Required. Keys of entities to look up.
     """
 
-    project_id = proto.Field(proto.STRING, number=8,)
-    read_options = proto.Field(proto.MESSAGE, number=1, message="ReadOptions",)
-    keys = proto.RepeatedField(proto.MESSAGE, number=3, message=entity.Key,)
+    project_id = proto.Field(
+        proto.STRING,
+        number=8,
+    )
+    read_options = proto.Field(
+        proto.MESSAGE,
+        number=1,
+        message="ReadOptions",
+    )
+    keys = proto.RepeatedField(
+        proto.MESSAGE,
+        number=3,
+        message=entity.Key,
+    )
 
 
 class LookupResponse(proto.Message):
@@ -81,13 +93,31 @@ class LookupResponse(proto.Message):
             resource constraints. The order of results in
             this field is undefined and has no relation to
             the order of the keys in the input.
+        read_time (google.protobuf.timestamp_pb2.Timestamp):
+            The time at which these entities were read or
+            found missing.
     """
 
-    found = proto.RepeatedField(proto.MESSAGE, number=1, message=gd_query.EntityResult,)
+    found = proto.RepeatedField(
+        proto.MESSAGE,
+        number=1,
+        message=gd_query.EntityResult,
+    )
     missing = proto.RepeatedField(
-        proto.MESSAGE, number=2, message=gd_query.EntityResult,
+        proto.MESSAGE,
+        number=2,
+        message=gd_query.EntityResult,
+    )
+    deferred = proto.RepeatedField(
+        proto.MESSAGE,
+        number=3,
+        message=entity.Key,
+    )
+    read_time = proto.Field(
+        proto.MESSAGE,
+        number=7,
+        message=timestamp_pb2.Timestamp,
     )
-    deferred = proto.RepeatedField(proto.MESSAGE, number=3, message=entity.Key,)
 
 
 class RunQueryRequest(proto.Message):
@@ -123,14 +153,31 @@ class RunQueryRequest(proto.Message):
             This field is a member of `oneof`_ ``query_type``.
     """
 
-    project_id = proto.Field(proto.STRING, number=8,)
-    partition_id = proto.Field(proto.MESSAGE, number=2, message=entity.PartitionId,)
-    read_options = proto.Field(proto.MESSAGE, number=1, message="ReadOptions",)
+    project_id = proto.Field(
+        proto.STRING,
+        number=8,
+    )
+    partition_id = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message=entity.PartitionId,
+    )
+    read_options = proto.Field(
+        proto.MESSAGE,
+        number=1,
+        message="ReadOptions",
+    )
     query = proto.Field(
-        proto.MESSAGE, number=3, oneof="query_type", message=gd_query.Query,
+        proto.MESSAGE,
+        number=3,
+        oneof="query_type",
+        message=gd_query.Query,
     )
     gql_query = proto.Field(
-        proto.MESSAGE, number=7, oneof="query_type", message=gd_query.GqlQuery,
+        proto.MESSAGE,
+        number=7,
+        oneof="query_type",
+        message=gd_query.GqlQuery,
     )
 
 
@@ -146,8 +193,16 @@ class RunQueryResponse(proto.Message):
             was set.
     """
 
-    batch = proto.Field(proto.MESSAGE, number=1, message=gd_query.QueryResultBatch,)
-    query = proto.Field(proto.MESSAGE, number=2, message=gd_query.Query,)
+    batch = proto.Field(
+        proto.MESSAGE,
+        number=1,
+        message=gd_query.QueryResultBatch,
+    )
+    query = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message=gd_query.Query,
+    )
 
 
 class BeginTransactionRequest(proto.Message):
@@ -162,9 +217,14 @@ class BeginTransactionRequest(proto.Message):
             Options for a new transaction.
     """
 
-    project_id = proto.Field(proto.STRING, number=8,)
+    project_id = proto.Field(
+        proto.STRING,
+        number=8,
+    )
     transaction_options = proto.Field(
-        proto.MESSAGE, number=10, message="TransactionOptions",
+        proto.MESSAGE,
+        number=10,
+        message="TransactionOptions",
     )
 
 
@@ -177,7 +237,10 @@ class BeginTransactionResponse(proto.Message):
             The transaction identifier (always present).
     """
 
-    transaction = proto.Field(proto.BYTES, number=1,)
+    transaction = proto.Field(
+        proto.BYTES,
+        number=1,
+    )
 
 
 class RollbackRequest(proto.Message):
@@ -193,8 +256,14 @@ class RollbackRequest(proto.Message):
             [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
     """
 
-    project_id = proto.Field(proto.STRING, number=8,)
-    transaction = proto.Field(proto.BYTES, number=1,)
+    project_id = proto.Field(
+        proto.STRING,
+        number=8,
+    )
+    transaction = proto.Field(
+        proto.BYTES,
+        number=1,
+    )
 
 
 class RollbackResponse(proto.Message):
@@ -248,10 +317,25 @@ class CommitRequest(proto.Message):
         TRANSACTIONAL = 1
         NON_TRANSACTIONAL = 2
 
-    project_id = proto.Field(proto.STRING, number=8,)
-    mode = proto.Field(proto.ENUM, number=5, enum=Mode,)
-    transaction = proto.Field(proto.BYTES, number=1, oneof="transaction_selector",)
-    mutations = proto.RepeatedField(proto.MESSAGE, number=6, message="Mutation",)
+    project_id = proto.Field(
+        proto.STRING,
+        number=8,
+    )
+    mode = proto.Field(
+        proto.ENUM,
+        number=5,
+        enum=Mode,
+    )
+    transaction = proto.Field(
+        proto.BYTES,
+        number=1,
+        oneof="transaction_selector",
+    )
+    mutations = proto.RepeatedField(
+        proto.MESSAGE,
+        number=6,
+        message="Mutation",
+    )
 
 
 class CommitResponse(proto.Message):
@@ -266,12 +350,25 @@ class CommitResponse(proto.Message):
         index_updates (int):
             The number of index entries updated during
             the commit, or zero if none were updated.
+        commit_time (google.protobuf.timestamp_pb2.Timestamp):
+            The transaction commit timestamp. Not set for
+            non-transactional commits.
     """
 
     mutation_results = proto.RepeatedField(
-        proto.MESSAGE, number=3, message="MutationResult",
+        proto.MESSAGE,
+        number=3,
+        message="MutationResult",
+    )
+    index_updates = proto.Field(
+        proto.INT32,
+        number=4,
+    )
+    commit_time = proto.Field(
+        proto.MESSAGE,
+        number=8,
+        message=timestamp_pb2.Timestamp,
     )
-    index_updates = proto.Field(proto.INT32, number=4,)
 
 
 class AllocateIdsRequest(proto.Message):
@@ -288,8 +385,15 @@ class AllocateIdsRequest(proto.Message):
             reserved/read-only.
     """
 
-    project_id = proto.Field(proto.STRING, number=8,)
-    keys = proto.RepeatedField(proto.MESSAGE, number=1, message=entity.Key,)
+    project_id = proto.Field(
+        proto.STRING,
+        number=8,
+    )
+    keys = proto.RepeatedField(
+        proto.MESSAGE,
+        number=1,
+        message=entity.Key,
+    )
 
 
 class AllocateIdsResponse(proto.Message):
@@ -303,7 +407,11 @@ class AllocateIdsResponse(proto.Message):
             with a newly allocated ID.
     """
 
-    keys = proto.RepeatedField(proto.MESSAGE, number=1, message=entity.Key,)
+    keys = proto.RepeatedField(
+        proto.MESSAGE,
+        number=1,
+        message=entity.Key,
+    )
 
 
 class ReserveIdsRequest(proto.Message):
@@ -323,9 +431,19 @@ class ReserveIdsRequest(proto.Message):
             auto-allocated.
     """
 
-    project_id = proto.Field(proto.STRING, number=8,)
-    database_id = proto.Field(proto.STRING, number=9,)
-    keys = proto.RepeatedField(proto.MESSAGE, number=1, message=entity.Key,)
+    project_id = proto.Field(
+        proto.STRING,
+        number=8,
+    )
+    database_id = proto.Field(
+        proto.STRING,
+        number=9,
+    )
+    keys = proto.RepeatedField(
+        proto.MESSAGE,
+        number=1,
+        message=entity.Key,
+    )
 
 
 class ReserveIdsResponse(proto.Message):
@@ -376,23 +494,50 @@ class Mutation(proto.Message):
             current version on the server, the mutation
             conflicts.
 
+            This field is a member of `oneof`_ ``conflict_detection_strategy``.
+        update_time (google.protobuf.timestamp_pb2.Timestamp):
+            The update time of the entity that this
+            mutation is being applied to. If this does not
+            match the current update time on the server, the
+            mutation conflicts.
+
             This field is a member of `oneof`_ ``conflict_detection_strategy``.
     """
 
     insert = proto.Field(
-        proto.MESSAGE, number=4, oneof="operation", message=entity.Entity,
+        proto.MESSAGE,
+        number=4,
+        oneof="operation",
+        message=entity.Entity,
     )
     update = proto.Field(
-        proto.MESSAGE, number=5, oneof="operation", message=entity.Entity,
+        proto.MESSAGE,
+        number=5,
+        oneof="operation",
+        message=entity.Entity,
     )
     upsert = proto.Field(
-        proto.MESSAGE, number=6, oneof="operation", message=entity.Entity,
+        proto.MESSAGE,
+        number=6,
+        oneof="operation",
+        message=entity.Entity,
     )
     delete = proto.Field(
-        proto.MESSAGE, number=7, oneof="operation", message=entity.Key,
+        proto.MESSAGE,
+        number=7,
+        oneof="operation",
+        message=entity.Key,
     )
     base_version = proto.Field(
-        proto.INT64, number=8, oneof="conflict_detection_strategy",
+        proto.INT64,
+        number=8,
+        oneof="conflict_detection_strategy",
+    )
+    update_time = proto.Field(
+        proto.MESSAGE,
+        number=11,
+        oneof="conflict_detection_strategy",
+        message=timestamp_pb2.Timestamp,
     )
 
 
@@ -412,15 +557,37 @@ class MutationResult(proto.Message):
             greater than the version of any previous entity
             and less than the version of any possible future
             entity.
+        update_time (google.protobuf.timestamp_pb2.Timestamp):
+            The update time of the entity on the server
+            after processing the mutation. If the mutation
+            doesn't change anything on the server, then the
+            timestamp will be the update timestamp of the
+            current entity. This field will not be set after
+            a 'delete'.
         conflict_detected (bool):
             Whether a conflict was detected for this
             mutation. Always false when a conflict detection
             strategy field is not set in the mutation.
     """
 
-    key = proto.Field(proto.MESSAGE, number=3, message=entity.Key,)
-    version = proto.Field(proto.INT64, number=4,)
-    conflict_detected = proto.Field(proto.BOOL, number=5,)
+    key = proto.Field(
+        proto.MESSAGE,
+        number=3,
+        message=entity.Key,
+    )
+    version = proto.Field(
+        proto.INT64,
+        number=4,
+    )
+    update_time = proto.Field(
+        proto.MESSAGE,
+        number=6,
+        message=timestamp_pb2.Timestamp,
+    )
+    conflict_detected = proto.Field(
+        proto.BOOL,
+        number=5,
+    )
 
 
 class ReadOptions(proto.Message):
@@ -444,6 +611,13 @@ class ReadOptions(proto.Message):
             transaction identifier is returned by a call to
             [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
 
+            This field is a member of `oneof`_ ``consistency_type``.
+        read_time (google.protobuf.timestamp_pb2.Timestamp):
+            Reads entities as they were at the given
+            time. This may not be older than 270 seconds.
+            This value is only supported for Cloud Firestore
+            in Datastore mode.
+
             This field is a member of `oneof`_ ``consistency_type``.
     """
 
@@ -454,9 +628,22 @@ class ReadOptions(proto.Message):
         EVENTUAL = 2
 
     read_consistency = proto.Field(
-        proto.ENUM, number=1, oneof="consistency_type", enum=ReadConsistency,
+        proto.ENUM,
+        number=1,
+        oneof="consistency_type",
+        enum=ReadConsistency,
+    )
+    transaction = proto.Field(
+        proto.BYTES,
+        number=2,
+        oneof="consistency_type",
+    )
+    read_time = proto.Field(
+        proto.MESSAGE,
+        number=4,
+        oneof="consistency_type",
+        message=timestamp_pb2.Timestamp,
     )
-    transaction = proto.Field(proto.BYTES, number=2, oneof="consistency_type",)
 
 
 class TransactionOptions(proto.Message):
@@ -496,14 +683,38 @@ class TransactionOptions(proto.Message):
                 being retried.
         """
 
-        previous_transaction = proto.Field(proto.BYTES, number=1,)
+        previous_transaction = proto.Field(
+            proto.BYTES,
+            number=1,
+        )
 
     class ReadOnly(proto.Message):
         r"""Options specific to read-only transactions.
+
+        Attributes:
+            read_time (google.protobuf.timestamp_pb2.Timestamp):
+                Reads entities at the given time.
+                This may not be older than 60 seconds.
         """
 
-    read_write = proto.Field(proto.MESSAGE, number=1, oneof="mode", message=ReadWrite,)
-    read_only = proto.Field(proto.MESSAGE, number=2, oneof="mode", message=ReadOnly,)
+        read_time = proto.Field(
+            proto.MESSAGE,
+            number=1,
+            message=timestamp_pb2.Timestamp,
+        )
+
+    read_write = proto.Field(
+        proto.MESSAGE,
+        number=1,
+        oneof="mode",
+        message=ReadWrite,
+    )
+    read_only = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        oneof="mode",
+        message=ReadOnly,
+    )
 
 
 __all__ = tuple(sorted(__protobuf__.manifest))
